----------------------------------------------------------------------------------
-- Company: 
-- Engineer:			befro
-- 
-- Create Date:    09:57:55 03/16/2009 
-- Design Name: 
-- Module Name:    PollinSerialRam - Behavioral 
-- Project Name: 
-- Target Devices: pollin cpld board
-- Tool versions: 
-- Description: 	zyklisches Ausgeben der ersten paar Daten im Ram auf
--						die serielle Schnittstelle ( 9600 Baud ) und die TestLEDs
-- Dependencies: 
--
-- Revision: 
-- Revision 0.01 - File Created
-- Additional Comments: 
--
----------------------------------------------------------------------------------
library IEEE;
use IEEE.STD_LOGIC_1164.ALL;
use IEEE.STD_LOGIC_ARITH.ALL;
use IEEE.STD_LOGIC_UNSIGNED.ALL;

---- Uncomment the following library declaration if instantiating
---- any Xilinx primitives in this code.
--library UNISIM;
--use UNISIM.VComponents.all;

entity chPollinSerial is
    Port ( clkIn : in  STD_LOGIC;
           serOut : out  STD_LOGIC;
           ledsOut : out  STD_LOGIC_VECTOR (7 downto 0);
			  ramAddress: out STD_LOGIC_VECTOR (16 downto 0 );
			  ramData: in STD_LOGIC_VECTOR (7 downto 0 );
			  ramNWE: out STD_LOGIC;
			  ramNOE: out STD_LOGIC );
end chPollinSerial;

architecture Behavioral of chPollinSerial is
	signal counter: STD_LOGIC_VECTOR ( 28 downto 0 );
	signal shiftRegister: STD_LOGIC_VECTOR ( 10 downto 0 );
	signal shiftClk: STD_LOGIC ;
	signal state: STD_LOGIC_VECTOR ( 3 downto 0 );
	signal clkDivider: STD_LOGIC_VECTOR ( 10 downto 0 );
	signal serialData: STD_LOGIC_VECTOR (7 downto 0 );
	
	-- ram signals
	signal ramAddressCounter: STD_LOGIC_VECTOR ( 16 downto 0 );
begin

	ramNOE <= '0' ; -- ram data output enable
	ramNWE <= '1' ; -- read ram
	
	serialClkDivider: process (clkIn)
	begin
			if RISING_EDGE(clkIn) then
				if(clkDivider =  1667) then -- divider = 1667 == 9600Baud@16MHz
					--clkDivider <= "00000000000";
					clkDivider <= conv_std_logic_vector(0,clkDivider'length); -- clkDivider auf 0 setzen
					shiftClk <= '1';
				else
					clkDivider <= clkDivider + 1;
					shiftClk <= '0';
				end if;
			end if;
	end process serialClkDivider;
	
	divider: process (clkIn)
	begin
		if RISING_EDGE(clkIn) then 
			counter <= counter+1;
		end if ;
	end process divider ;
	
	shifter: process ( shiftClk )
	begin
		if RISING_EDGE (shiftClk) then
			case state is
				when "0000" => shiftRegister <= '1' & serialData & "01"; -- stop bit + daten + startbit
				when others => shiftRegister <= '1' & shiftRegister(10 downto 1); -- daten rausschieben
			end case;
		end if;
	end process shifter;

	ramAddressGenerator: process (counter(21))
	begin
		if RISING_EDGE (counter(21)) then
				if(ramAddressCounter =  4) then -- endAdresse fr Ram Bereich, willkrlich klein gewhlt
					ramAddressCounter <= conv_std_logic_vector(0,ramAddressCounter'length); -- ramAddressCounter auf 0 setzen
				else
					ramAddressCounter <= ramAddressCounter + 1;
				end if;
		end if;
	end process ramAddressGenerator;
		
	--serialData <= counter(28 downto 21 );
	serialData <= ramData; -- Ram Daten auf serielle Schnittstelle
	state <= counter(20 downto 17);

	ramAddress <= ramAddressCounter;
	--ledsOut <= serialData;
	ledsOut <= ramData;
	
	serOut <= shiftRegister(0);
end Behavioral;

